import { Request, Response } from "express";
import prisma, { Prisma } from "@server/prisma";
import { AjaxData, AjaxResponse } from "datatables.net-dt";

export const get = async (request: Request, response: Response) => {
    if (!request.query.id) {
        response.status(400).json({ error: "missing 'id' query" });
        return;
    }

    const feed = await prisma.feed.findUnique({
        where: { id: Number(request.query.id) },
        include: { channels: true }
    });

    if (!feed) {
        response.status(404).json({ message: "no result found" });
        return;
    }

    response.json(feed);
};

export const post = async (request: Request, response: Response) => {
    const guildId = request.params.guildId;
    const { name, url, active, channels } = request.body;

    let feed;

    try {
        feed = await prisma.feed.create({
            data: { 
                name: name,
                url: url,
                guild_id: guildId,
                active: active,
                channels: channels
             }
        });
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(201).json(feed);
};

export const patch = async () => {}  // TODO ...

export const del = async (request: Request, response: Response) => {
    const { ids } = request.body;
    const guildId = request.params.guildId;

    if (!ids || !Array.isArray(ids)) {
        response.status(400).json({ error: "invalid request body" });
        return;
    }

    try {
        await prisma.feed.deleteMany({ where: {
            id: { in: ids },
            guild_id: guildId
        }});
    }
    catch (error) {
        console.error(error);
        if (error instanceof Prisma.PrismaClientKnownRequestError) {
            response.status(500).json({ error: error.message });
            return;
        }
    }

    response.status(204).json(null);
};


interface DatatableQuery extends AjaxData {
    filters: { [key: string]: any };
}

export const datatable = async (request: Request, response: Response) => {
    const query = request.body as unknown as DatatableQuery;

    const orderBy: Prisma.FeedOrderByWithRelationInput = query.order?.length
        ? { [query.columns[query.order[0].column].data]: query.order[0].dir }
        : { id: "asc" };

    const where: Prisma.FeedWhereInput = query.search?.value
        ? {
            OR: Object.values(query.columns)
                .filter(col => col.searchable)
                .map(col => ({
                    [col.data]: { contains: query.search.value }
                })) as Prisma.FeedWhereInput[]
        }
        : {};

    const data = await prisma.feed.findMany({
        skip: query.start,
        take: query.length,
        orderBy: orderBy,
        where: where,
        include: { channels: true },
    });

    response.json(<AjaxResponse>{
        data: data,
        recordsFiltered: await prisma.feed.count({ where: where }),
        recordsTotal: await prisma.feed.count(),
        draw: query.draw
    });
};

export default { get, post, patch, del, datatable };